module ChainedValidation
  class Validate
    def initialize(validations)
      @validations = validations
    end

    # Return a result of validation.
    def claim
      is_valid?
    end

    # Claim result with exceptions.
    def claim!
      is_valid? ? true : result_exceptions
    end

    private

    # Parse ChainedValidation::Validation classes for one array of hashes with results:
    # [{ :is_boolean=>{:object=>"foo", :result=>false}}, {:is_boolean=>{:object=>true, :result=>true }}]
    def results
      @validations.map(&:result).collect { |result| result.values.first }
    end

    # Return only value of results in an array of booleans:
    # [false, true]
    def results_values
      results.collect { |result| result[:result] }
    end

    # Return a hash for validation false.
    # [{:is_boolean=>{ :object=>"foo", :result=>false }}]
    def results_false
      @validations.map(&:result).collect { |result| result if result.values.first[:result] == false }.compact
    end

    # Collect all error messages.
    def results_messages
      results_false.map do |result|
        "Object #{result.values.first[:object]} does not match the rule #{result.keys.first}"
      end
    end

    # Return an exception for a list of validations with result false.
    def result_exceptions
      results_messages.each { |msg| raise TypeError, msg }
    end

    # If in result_values have an false value, the validation of rules failed.
    def is_valid?
      results_values.include?(false) ? false : true
    end
  end
end
